//==========================================================================
// File: demo.c
// Description: An interactive demonstration of routing algorithms
// Author: Pat Morin
// Created: Mon Mar 29 19:09:25 EST 1999
//==========================================================================
#include <LEDA/list.h>
#include <LEDA/point.h>
#include <LEDA/random_source.h>
#include <LEDA/graph.h>
#include <LEDA/window.h>

#include "mesh.h"
#include "ksu.h"
#include "facetree.h"

int main(int argc, char *argv[]) 
{
  node v;
  edge e;

  // read command line arguments
  if (argc != 2) {
    cerr << "Usage: " << argv[0] << " <n>" << endl;
    exit(1);
  }
  int n = atoi(argv[1]);

  // create our persistent data structures
  random_source R;
  GRAPH<point,int> G;

  // open the display window
  window W(300+10, 300+20, "online routing demo");  
  W.button("dismiss",1);
  W.open(window::center,window::center);
  W.init(-.1, 1.2, -.1);
    
  // create our graph
  do {
    // create the full graph
    list<point> L;
    double x, y;
    for (int i = 0; i < n; i++) {
      R >> x;
      R >> y;
      L.append(point(x, y));
    }
    MESH(n, G);

    // generate random edge faults
    edge enext = G.first_edge();
    while (enext != nil) {
      e = enext;
      enext = G.succ_edge(enext);
      double coin;
      R >> coin;
      if (coin > 1.8) {
	if (enext == G.reversal(e) && enext != nil) {
	  enext = G.succ_edge(enext);
	}
	if (G.reversal(e) != nil)
	  G.del_edge(G.reversal(e));
	G.del_edge(e);
      }
    }
  } while (!Is_Connected(G));

  // clear the window
  W.clear();

  // draw the graph
  forall_edges (e, G) {
    W.draw_segment(G[G.source(e)], G[G.target(e)], black);
  }

  // draw the nodes of the graphs in blue
  W.set_node_width(2);
  forall_nodes (v, G) {
    W.draw_filled_node(G[v], blue);
  }

  // setup the position array for our path finding algorithms
  node_array<point> pos(G);
  forall_nodes (v, G) {
    pos[v] = G[v];
  }

  node s, t;
  bool half_done = false;
  while (1 < 2) {
    point p;
    int k = W.read_mouse(p);
    switch (k) {
    case MOUSE_BUTTON(1):
      if (!half_done) {
	node vmin = G.first_node();
	forall_nodes (v, G) {
	  if (G[v].distance(p) < G[vmin].distance(p)) {
	    vmin = v;
	  }
	}
	if (G[vmin].distance(p) < 0.05) {
	  s = vmin;
	  half_done = true;
	}
      } else {
	node vmin = G.first_node();
	forall_nodes (v, G) {
	  if (G[v].distance(p) < G[vmin].distance(p)) {
	    vmin = v;
	  }
	}
	if (G[vmin].distance(p) < 0.05) {
	  t = vmin;
	  list<edge> P;
	  FACETREE_PATH(G, pos, s, t, P);
	  half_done = false;
	}
      }
      break;

    case 1:
      return 0;
    }
  }
  return 0;
}
