//==========================================================================
// File: mesh.h
// Description: A function for generating a 2d square mesh as a planar map 
// Author: Pat Morin
// Created: 
//==========================================================================
#include <math.h>
#include <LEDA/plane_alg.h>


#include "mesh.h"

//==========================================================================
// Make G into a planar map corresponding to an nxn mesh.
// Complexity: O(n^2 log n)
//==========================================================================
void MESH(int n, GRAPH<point,int> &G)
{
  n = (int)ceil(sqrt(n));
  
  // create a delaunay triangulation of lattice points
  list<point> L;
  double space = 1.0/(n-1);
  for (int i = 0; i < n; i++) {
    for (int j = 0; j < n; j++) {
      L.append(point(space*i, space*j));
    }
  }
  DELAUNAY_TRIANG(L, G);

  // delete the diagonals
  edge e;
  edge enext = G.first_edge();
  while (enext != nil) {
    e = enext;
    enext = G.succ_edge(enext);
    if (G[G.source(e)].xcoord() != G[G.target(e)].xcoord()
	&& G[G.source(e)].ycoord() != G[G.target(e)].ycoord()) {
      if (enext == G.reversal(e) && enext != nil) {
	enext = G.succ_edge(enext);
      }
      if (G.reversal(e) != nil)
	G.del_edge(G.reversal(e));
      G.del_edge(e);
    }
  }
}

